library(Asp)
asp.dbConnect("mclatent","ralmond")

initial <- daCapo(window=c(1000,6000))

daCapoApplyPrint(initial,summary)

iparams <- unlist(c(initial$stu,initial$link),recursive=FALSE)

lapply(iparams, function (m) {gelman.diag(m[,1],transform=TRUE)})
gelman.diag(iparams[[4]],transform=TRUE)

lapply(iparams, function (m) {autocorr.diag(m[,1],lags=5)})

maxRNC(unlist(initial$linkModels,recursive=FALSE))


lam0 <- getAllParameters("lc2",iterations=c(1,2000))
lam0 <- lam0[[1]][,1]
traceAutoplot(lam0)
gelman.plot(lam0)


## Paste them all together into one big table for convenience
lc2 <- iparams[[1]][,1]
#varnames(iparams) <- "theta"
Item1 <- iparams[[2]][,c(3,1)]
varnames(Item1) <- c("pi.10","pi.11")
Item2 <- iparams[[3]][,c(3,1)]
varnames(Item2) <- c("pi.20","pi.21")
Item3 <- iparams[[4]][,6:1]
varnames(Item3) <- c("pi.300","pi.301", "pi.302","pi.310","pi.311","pi.312")

allmcmcs <- lapply(1:nchain(lc2), function(chain) {
  mcmc(cbind(lambda=lc2[[chain]], Item1[[chain]],
             Item2[[chain]], Item3[[chain]]),
       start=start(lc2), end=end(lc2), thin=thin(lc2))
})
allInitialParams <- do.call("mcmc.list",allmcmcs)
sumInitial <- summary(allInitialParams)

xtable(allInitialParams[[1]][1000*(0:5)+1,],digits=rep(2,12))
xtable(t(sumInitial$stat),digits=rep(3,12))
xtable(t(sumInitial$q),digits=rep(2,12))

selectedInitialParams <- allInitialParams[,c(1,3,4,6:8)]

selectedInitialPooled <- mcmc(rbind(selectedInitialParams[[1]],
                              selectedInitialParams[[2]],
                              selectedInitialParams[[3]]))

pdf("../figs/MCInitialPost.pdf",width=3.5,height=4.5)
par(mfrow=c(3,2))
densplot(selectedInitialPooled,ylim=c(0,5.5))
dev.off()
system("open ../figs/MCInitialPost.pdf")

postscript("../figs/MCInitialPost.eps",width=3.5,height=4.5,
    onefile=FALSE,paper="special",horizontal=FALSE)
par(mfrow=c(3,2))
densplot(selectedInitialPooled,ylim=c(0,5.5))
dev.off()
system("open ../figs/MCInitialPost.eps")

